rm(list=ls())
library(foreign)
library(xtable)
dat =read.dta("/disk/agedisk5/medicare.work/afinkels-DUA22559/schrimpf/Data/extract/data_extract.dta")
figureDirectory <- "figures"
tableDirectory <- "tables"

## Settings
lambda <- 0.45 ## strenght of results highly depend on lambda. choose carefully
nbin <- 50 ## number of bins for distribution of theta and p

## data processing
dat$bnftphas <- factor(dat$bnftphas)
# sort by id, claim date
dat <- dat[order(dat$bene_id,dat$srvc_dt),]
## collapse to weekly data
dat$week <- as.integer( (as.numeric(dat$srvc_dt) -
                         as.numeric(as.Date(sprintf("%4d-01-01",dat$yr))))/7)
dat$joinweek <- as.integer((as.numeric(as.Date(sprintf("%4d-%2d-01",dat$yr,dat$partd_joinmonth)))
                            - as.numeric(as.Date(sprintf("%4d-01-01",dat$yr))) ) / 7)
## expected number of weeks eligible assuming uniform distribution of join week 
firstWk <- ave(dat$week,dat$bene_id,FUN=min)
nWk <- max(dat$week) +1
dat$eweeks <- (nWk-dat$joinweek)*(dat$joinweek==firstWk) +
  (nWk-(dat$joinweek+0.5))*(dat$joinweek==(firstWk-1)) +
  (nWk-(dat$joinweek+1))*(dat$joinweek==(firstWk-2)) +
  (nWk-(dat$joinweek+3/2))*(dat$joinweek<=(firstWk-3)) +
  (nWk - firstWk)*(dat$joinweek>firstWk)


dat$costWk <- ave(dat$totalcst,dat$bene_id,dat$week,FUN=sum)
dat$payWk <- ave(dat$ptpayamt,dat$bene_id,dat$week,FUN=sum)
wkdat <- dat[,c("bene_id","week","joinweek","partd_joinmonth","yr"
                ,"plan","ded_amt","icl_amt","oopt_amt","costWk","payWk","eweeks")]
wkdat <- wkdat[!duplicated(wkdat) & !is.na(wkdat$partd_joinmonth),]
## only use deductible plan
wkdat <- wkdat[wkdat$plan=="xxkSreD9",]
wkdat <- wkdat[order(wkdat$bene_id,wkdat$week),]
wkdat$cumPay <- ave(wkdat$payWk,wkdat$bene_id, wkdat$yr, FUN=cumsum)
wkdat$cumCost <- ave(wkdat$costWk,wkdat$bene_id,wkdat$yr,FUN=cumsum)
wkdat$segment <- as.factor(1*(wkdat$cumCost<wkdat$ded_amt) +
                           2*(wkdat$cumCost>=wkdat$ded_amt & wkdat$cumCost<wkdat$icl_amt) +
                           3*(wkdat$cumCost>=wkdat$icl_amt & wkdat$cumPay<wkdat$oopt_amt) +
                           4*(wkdat$cumPay>=wkdat$oopt_amt))

## estimate bounds for (unconditional) distribution of theta
theta <- c()
theta$grid <- quantile(wkdat$costWk,probs=seq(0,1,length.out=nbin+1))
names(theta$grid) <- c()
## G[theta|d=1,z]
gthetad1 <- sapply(1:12,
                   function(x) {
                     h <- hist(wkdat$costWk[wkdat$partd_joinmonth==x],
                               breaks=theta$grid,plot=FALSE)
                     cumsum(h$counts)/sum(h$counts)
                   })
## P(d=1|z)
pdi <- ave(wkdat$payWk,wkdat$bene_id,FUN=length)/wkdat$eweeks
newid <- c(TRUE,wkdat$bene_id[2:length(wkdat$bene_id)]!=
           wkdat$bene_id[1:(length(wkdat$bene_id)-1)])
lambdas <- c(0.45,0.5,0.6,0.7)
library(ggplot2)
theta$glo <- matrix(NA,nrow=nbin,ncol=length(lambdas))
theta$ghi <- matrix(NA,nrow=nbin,ncol=length(lambdas))
df <- data.frame()
mids <- (theta$grid[1:nbin]+theta$grid[2:(nbin+1)])/2
for (l in 1:length(lambdas)) {
  lambda <- lambdas[l]
  pd1 <- by(pdi,wkdat$partd_joinmonth, FUN=mean)/lambda
  gthpd1 <- gthetad1*matrix(rep(pd1,nbin),nrow=nbin,ncol=12,byrow=TRUE)
  theta$glo[,l] <- sapply(1:nbin,function(x) { max(gthpd1[x,]) } )
  theta$ghi[,l] <- sapply(1:nbin,function(x) { min(gthpd1[x,]+(1-pd1)) } )
  df <- rbind(df,data.frame(cbind(mids,theta$glo[,l],theta$ghi[,l],rep(lambda,nbin))))
}
names(df) <- c("grid","glo","ghi","lambda")
pdf(sprintf("%s/GthetaBnds.pdf",figureDirectory))
ggplot(df[df$grid<200,]) + geom_line(aes(x=grid, y=glo, color=factor(lambda), group=lambda)) +
  geom_line(aes(x=grid, y=ghi, color=factor(lambda),group=lambda)) +
  scale_color_brewer(palette="Set1",type="qual", name=expression(lambda)) +
  xlab(expression(theta)) + ylab("CDF") 
dev.off()

#################################################################################
## Bounds on value function
nx <- 19 ## number of knots for value function approximation
year <- 2009 ## for simplicity only use this year of data
T <- length(unique(wkdat$week))
delta <- 0.95^(1/T)

vdat <- wkdat[wkdat$yr==2009,]
## balance panel
vdat <- reshape(vdat,timevar="week",v.names=c("costWk","payWk"),idvar="bene_id",
                drop=c("cumCost","cumPay","segment"),direction="wide")
vdat <- reshape(vdat,timevar="week",v.names=c("costWk","payWk"),idvar="bene_id",
                direction="long")
vdat <- vdat[order(vdat$bene_id,vdat$week),]
cumsum.na <- function(x) { y <- x
                           y[is.na(x)] <- 0
                           cumsum(y)
                         }
vdat$cumPay <- ave(vdat$payWk,vdat$bene_id, vdat$yr,
                   FUN=cumsum.na)
vdat$cumCost <- ave(vdat$costWk,vdat$bene_id,vdat$yr,
                    FUN=cumsum.na)
vdat$segment <- as.factor(1*(vdat$cumCost<vdat$ded_amt) +
                           2*(vdat$cumCost>=vdat$ded_amt & vdat$cumCost<vdat$icl_amt) +
                           3*(vdat$cumCost>=vdat$icl_amt & vdat$cumPay<vdat$oopt_amt) +
                           4*(vdat$cumPay>=vdat$oopt_amt))

## knots for value function approximation
val <- c()
qcut <- seq(0,1,length.out=nx+1)
val$xcut <- quantile(wkdat$cumCost,qcut)
val$x <- quantile(wkdat$cumCost,qcut[1:nx]+qcut[1]/2)
val$vxhi <- array(NA,dim=c(nx,length(lambda),T))
val$vxlo <- array(NA,dim=c(nx,length(lambda),T))
val$vhi <- list()
val$vlo <- list()
for (l in 1:length(lambdas)) {
  val$vhi[[l]] <- list()
  val$vlo[[l]] <- list()
}
t <- 1
l <- 1
lambda <- lambdas[l]
maxoop <- ave(vdat$payWk,vdat$segment,
              FUN=function(x) {
                quantile(x,0.85,na.rm=TRUE)
              } )
yhi <- -vdat$payWk
yhi[is.na(yhi)] <- 0              
ylo <- -vdat$payWk
ylo[is.na(ylo)] <- -maxoop[is.na(ylo)]
ylo <- ylo + (1-lambda)*maxoop
val$vxhi[,l,t] <- lambda*sapply(1:nx,
                         function(x) {
                           mean(yhi[vdat$cumCost>=val$xcut[x] & vdat$cumCost<val$xcut[x+1] &
                                    vdat$week==T-t])/lambda
                         })

val$vxlo[,l,t] <- lambda*sapply(1:nx, function(x) {
  mz <- sapply(1:12, function(z) {
    mean(ylo[vdat$cumCost>=val$xcut[x] & vdat$cumCost<val$xcut[x+1] &
             vdat$week==T-t & vdat$partd_joinmonth==z])/lambda })
  max(mz[!is.na(mz)]) }  )

val$vhi[[l]][[t]] <- splinefun(val$x,val$vxhi[,l,t],method="fmm")
val$vlo[[l]][[t]] <- splinefun(val$x,val$vxlo[,l,t],method="fmm")

theta <- vdat$costWk
theta[is.na(theta)] <- 0 ## won't get used anyway
for (t in 2:T) {
  yhi <- -vdat$payWk + delta*val$vhi[[l]][[t-1]](vdat$cumCost+theta)
  yhi[is.na(yhi)] <- 0 + delta*val$vhi[[l]][[t-1]](vdat$cumCost[is.na(yhi)])
  ylo <- -vdat$payWk + delta*val$vlo[[l]][[t-1]](vdat$cumCost+theta)
  ylo[is.na(ylo)] <- -maxoop[is.na(ylo)] + delta*val$vlo[[l]][[t-1]](vdat$cumCost[is.na(ylo)])
  ylo <- ylo + (1-lambda)*maxoop

  val$vxhi[,l,t] <- (1-lambda)*delta*val$vhi[[l]][[t-1]](val$x) +
    lambda*sapply(1:nx,function(x) {
      mean(yhi[vdat$cumCost>=val$xcut[x] & vdat$cumCost<val$xcut[x+1] &
               vdat$week==T-t])/lambda } )
    
  val$vxlo[,l,t] <- (1-lambda)*delta*val$vlo[[l]][[t-1]](val$x)
    lambda*sapply(1:nx, function(x) {
      mz <- sapply(1:12, function(z) {
        mean(ylo[vdat$cumCost>=val$xcut[x] & vdat$cumCost<val$xcut[x+1] &
                 vdat$week==T-t & vdat$partd_joinmonth==z])/lambda })
      max(mz[!is.na(mz)]) }  )
  val$vhi[[l]][[t]] <- splinefun(val$x,val$vxhi[,l,t],method="fmm")
  val$vlo[[l]][[t]] <- splinefun(val$x,val$vxlo[,l,t],method="fmm")

  print(sprintf("%d of %d",t,T))
}

